package top.huic.trtc_cloud_plugin.view;

import android.content.Context;
import android.view.SurfaceView;
import android.view.View;
import androidx.annotation.NonNull;

import com.tencent.rtmp.ui.TXCloudVideoView;
import com.tencent.trtc.TRTCCloud;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.StandardMessageCodec;
import io.flutter.plugin.platform.PlatformView;
import io.flutter.plugin.platform.PlatformViewFactory;
import top.huic.trtc_cloud_plugin.util.CommonUtil;

/**
 * 视频视图
 */
public class TRTCCloudVideoPlatformView extends PlatformViewFactory
        implements PlatformView, MethodChannel.MethodCallHandler {

    /**
     * 全局标识
     */
    public static final String SIGN = "trtcCloudChannelView";

    /**
     * 日志标签
     */
    private static final String TAG = TRTCCloudVideoPlatformView.class.getName();

    /**
     * 消息器
     */
    private BinaryMessenger messenger;

    /**
     * 腾讯云音视频通信实例
     */
    private TRTCCloud trtcCloud;

    /**
     * 腾讯云视频视图
     */
    private TXCloudVideoView remoteView;

    private SurfaceView mSurfaceView;

    /**
     * 初始化工厂信息，此处的域是 PlatformViewFactory
     */
    public TRTCCloudVideoPlatformView(Context context, BinaryMessenger messenger) {
        super(StandardMessageCodec.INSTANCE);

        this.messenger = messenger;
        trtcCloud = TRTCCloud.sharedInstance(context);
        mSurfaceView = new SurfaceView(context);
        remoteView = new TXCloudVideoView(mSurfaceView);
    }

    @Override
    public View getView() {
//        return remoteView;
        return mSurfaceView;
    }

    @Override
    public void dispose() {
    }

    @Override
    public PlatformView create(Context context, int viewId, Object args) {
        // 每次实例化对象，保证界面上每一个组件的独立性(此处域是 PlatformView和MethodChannel.MethodCallHandler)
        TRTCCloudVideoPlatformView view = new TRTCCloudVideoPlatformView(context, messenger);
        new MethodChannel(messenger, SIGN + "_" + viewId).setMethodCallHandler(view);
        return view;
    }

    @Override
    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        switch (call.method) {
            case "startRemoteView":
                this.startRemoteView(call, result);
                break;
            case "startLocalPreview":
                this.startLocalPreview(call, result);
                break;
            default:
                result.notImplemented();
        }

    }

    /**
     * 开启远程显示,指定要观看 userId 的视频流类型
     */
    private void startRemoteView(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = CommonUtil.getParam(call, result, "userId");
        int streamType = CommonUtil.getParam(call, result, "streamType");
        trtcCloud.startRemoteView(userId, streamType, this.remoteView);
        result.success(null);
    }

    /**
     * 开启本地视频采集
     */
    private void startLocalPreview(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        boolean frontCamera = CommonUtil.getParam(call, result, "frontCamera");
        trtcCloud.startLocalPreview(frontCamera, this.remoteView);
        result.success(null);
    }
}