import 'dart:ui';
import 'dart:async';
import 'dart:convert';
import 'package:flutter/services.dart';
import 'package:flutter/material.dart';
import 'package:dio/dio.dart';
import 'package:toast/toast.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:trtc_cloud_plugin_example/models/meeting.dart';
import 'package:provider/provider.dart';

Color timBaseColor = Color(int.parse('006fff', radix: 16)).withAlpha(255);

class LoginPage extends StatefulWidget {
  @override
  _LoginPageState createState() => _LoginPageState();
}

class _LoginPageState extends State<LoginPage> {
  String phone = ''; //手机号
  String code = ''; //验证码
  String sessionId = ''; // 获取验证码后后台下发的sessionId
  Timer _timer;
  int _countdownTime = 0;
  var meetModel;

  @override
  initState() {
    super.initState();
    meetModel = context.read<MeetingModel>();
    initData();
  }

  initData() async {
    try {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      String userStr = prefs.getString('TRTC_FLUTTER_TOKEN');
      if (userStr != null) {
        Map userMap = jsonDecode(userStr);
        loginWidthToken(userMap);
      }
    } catch (e) {}
  }

  // 提示浮层
  showToast(text) {
    Toast.show(
      text,
      context,
      duration: Toast.LENGTH_SHORT,
      gravity: Toast.CENTER,
    );
  }

  // 倒计时定时器
  void startCountdownTimer() {
    const oneSec = const Duration(seconds: 1);
    var callback = (timer) => {
          setState(() {
            if (_countdownTime < 1) {
              _timer.cancel();
            } else {
              _countdownTime = _countdownTime - 1;
            }
          })
        };

    _timer = Timer.periodic(oneSec, callback);
  }

  // 获取验证码
  getLoginCode(context) async {
    if (phone.length == 0) {
      showToast('请输入手机号');
      return;
    } else if (!new RegExp(r"1[0-9]\d{9}$").hasMatch(phone)) {
      showToast('手机号格式错误');
      return;
    } else {
      Response response = await new Dio().get(
          "https://service-c2zjvuxa-1252463788.gz.apigw.tencentcs.com/release/sms",
          queryParameters: {
            "method": "getSms",
            "phone": phone,
          });
      if (response.data['errorCode'] == 0) {
        showToast('验证码发送成功');
        sessionId = response.data['data']['sessionId'];
        setState(() {
          _countdownTime = 60;
        });
        //开始倒计时
        startCountdownTimer();
      } else {
        showToast(response.data['errorMessage']);
      }
    }
  }

  // 根据token自动登录
  loginWidthToken(userMap) async {
    Response response = await new Dio().get(
        "https://service-c2zjvuxa-1252463788.gz.apigw.tencentcs.com/release/sms",
        queryParameters: {
          "method": "login",
          "phone": userMap['phone'],
          "token": userMap['token']
        });
    if (response.data['errorCode'] == 0) {
      showToast('登录成功');
      SharedPreferences prefs = await SharedPreferences.getInstance();
      await prefs.setString(
          'TRTC_FLUTTER_TOKEN', jsonEncode(response.data['data']));
      meetModel.setUserInfo(response.data['data']);

      Navigator.pushNamed(context, "/index");
    } else {
      showToast(response.data['errorMessage']);
    }
  }

  // 根据验证码校验登录
  login() async {
    if (sessionId == '') {
      showToast('请先获取验证码');
      return;
    } else if (!new RegExp(r"1[0-9]\d{9}$").hasMatch(phone)) {
      showToast('手机号格式错误');
      return;
    } else if (code == '') {
      showToast('请输入验证码');
      return;
    }
    Response response = await new Dio().get(
        "https://service-c2zjvuxa-1252463788.gz.apigw.tencentcs.com/release/sms",
        queryParameters: {
          "method": "login",
          "phone": phone,
          "sessionId": sessionId,
          "code": code
        });
    if (response.data['errorCode'] == 0) {
      showToast('登录成功');
      SharedPreferences prefs = await SharedPreferences.getInstance();
      await prefs.setString(
          'TRTC_FLUTTER_TOKEN', jsonEncode(response.data['data']));
      meetModel.setUserInfo(response.data['data']);
      Navigator.pushNamed(context, "/index");
    } else {
      showToast(response.data['errorMessage']);
    }
  }

  @override
  dispose() {
    super.dispose();
    if (_timer != null) {
      _timer.cancel();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: Container(
        color: Color.fromRGBO(14, 25, 44, 1),
        padding: const EdgeInsets.all(15.0),
        height: MediaQuery.of(context).size.height,
        child: Column(
          children: <Widget>[
            Container(
              margin:
                  EdgeInsets.only(top: MediaQuery.of(context).padding.top + 20),
              child: Text('腾讯云TRTC',
                  textAlign: TextAlign.center,
                  style: TextStyle(color: Colors.white, fontSize: 20)),
            ),
            Container(
              height: 180,
              width: MediaQuery.of(context).size.width,
              decoration: BoxDecoration(
                  image: DecorationImage(
                fit: BoxFit.cover,
                image: AssetImage("images/bg_main_title.png"),
              )),
            ),
            Container(
              color: Color.fromRGBO(13, 44, 91, 1),
              padding: const EdgeInsets.only(left: 10.0, right: 10.0),
              child: Form(
                child: Column(
                  children: <Widget>[
                    TextField(
                      style: TextStyle(color: Colors.white),
                      autofocus: false,
                      decoration: InputDecoration(
                        labelText: "手机号",
                        hintText: "请输入手机号码",
                        labelStyle: TextStyle(color: Colors.white),
                        hintStyle: TextStyle(
                            color: Color.fromRGBO(255, 255, 255, 0.5)),
                        border: InputBorder.none,
                      ),
                      keyboardType: TextInputType.number,
                      onChanged: (value) {
                        setState(() {
                          phone = value;
                        });
                      },
                    ),
                    Flex(
                      direction: Axis.horizontal,
                      children: <Widget>[
                        Expanded(
                          child: TextField(
                            style: TextStyle(color: Colors.white),
                            decoration: InputDecoration(
                              labelText: "验证码",
                              hintText: "请输入验证码",
                              labelStyle: TextStyle(color: Colors.white),
                              hintStyle: TextStyle(
                                  color: Color.fromRGBO(255, 255, 255, 0.5)),
                              border: InputBorder.none,
                            ),
                            keyboardType: TextInputType.number,
                            //校验密码
                            onChanged: (v) {
                              setState(() {
                                code = v;
                              });
                            },
                          ),
                          flex: 2,
                        ),
                        Expanded(
                          child: _countdownTime > 0
                              ? Text(_countdownTime.toString() + '秒',
                                  textAlign: TextAlign.center,
                                  style: TextStyle(
                                      color: Color.fromRGBO(0, 110, 253, 1),
                                      fontSize: 16))
                              : GestureDetector(
                                  child: Container(
                                    alignment: Alignment.center,
                                    width: 100.0,
                                    height: 50,
                                    child: Text(
                                      '获取验证码',
                                      style: TextStyle(
                                          color: Color.fromRGBO(0, 110, 253, 1),
                                          fontSize: 16),
                                    ),
                                  ),
                                  onTap: () => getLoginCode(context),
                                ),
                          flex: 1,
                        )
                      ],
                    ),
                  ],
                ),
              ),
            ),
            Container(
              margin: const EdgeInsets.only(top: 40.0),
              child: Row(
                children: <Widget>[
                  Expanded(
                    child: RaisedButton(
                      padding: EdgeInsets.all(15.0),
                      child: Text(
                        "登录",
                        style: TextStyle(fontSize: 16),
                      ),
                      color: timBaseColor,
                      textColor: Colors.white,
                      onPressed: () async {
                        login();
                        // Navigator.pushNamed(context, '/index');
                      },
                    ),
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}
