import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:trtc_cloud_plugin/TRTCCloudDef.dart';
import 'package:toast/toast.dart';
import 'package:permission_handler/permission_handler.dart';

class IndexPage extends StatefulWidget {
  IndexPage({Key key}) : super(key: key);

  @override
  State<StatefulWidget> createState() => IndexPageState();
}

class IndexPageState extends State<IndexPage> {
  /// 用户id
  String userId;

  /// 登录后签名
  String userSig;

  /// 会议id
  String meetId = '256';

  /// 用户名
  String userName = 'clavie';

  /// 是否开启摄像头
  bool enabledCamera = true;

  /// 是否开启麦克风
  bool enabledMicrophone = false;

  /// 音质选择
  int quality = TRTCCloudDef.TRTC_AUDIO_QUALITY_SPEECH;

  // 提示浮层
  showToast(text) {
    Toast.show(
      text,
      context,
      duration: Toast.LENGTH_SHORT,
      gravity: Toast.CENTER,
    );
  }

  @override
  initState() {
    super.initState();
  }

  enterMeeting() async {
    if (meetId == '') {
      showToast('请输入会议号');
      return;
    } else if (meetId == '0') {
      showToast('请输入合法的会议ID');
      return;
    } else if (!new RegExp(r"\d+").hasMatch(meetId)) {
      showToast('请输入合法的会议ID');
      return;
    }
    if (userName == '') {
      showToast('请输入用户名');
      return;
    }
    if (await Permission.camera.request().isGranted &&
        await Permission.microphone.request().isGranted) {
      Navigator.pushNamed(context, "/video", arguments: {
        "meetId": int.parse(meetId),
        "userName": userName,
        "enabledCamera": enabledCamera,
        "enabledMicrophone": enabledMicrophone,
        "quality": quality
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('多人视频会议'),
        centerTitle: true,
        elevation: 0,
        backgroundColor: Color.fromRGBO(14, 25, 44, 1),
      ),
      body: Container(
        color: Color.fromRGBO(14, 25, 44, 1),
        padding: const EdgeInsets.symmetric(horizontal: 20.0),
        child: Column(
          children: <Widget>[
            Container(
              color: Color.fromRGBO(13, 44, 91, 1),
              margin: const EdgeInsets.only(top: 30.0),
              padding: const EdgeInsets.only(left: 10.0, right: 10.0),
              child: Column(
                children: [
                  TextField(
                      style: TextStyle(color: Colors.white),
                      autofocus: false,
                      decoration: InputDecoration(
                        labelText: "会议号",
                        hintText: "请输入会议号",
                        labelStyle: TextStyle(color: Colors.white),
                        hintStyle: TextStyle(
                            color: Color.fromRGBO(255, 255, 255, 0.5)),
                        enabledBorder: UnderlineInputBorder(
                          borderSide: BorderSide(color: Colors.white),
                        ),
                      ),
                      keyboardType: TextInputType.number,
                      onChanged: (value) => meetId = value),
                  TextField(
                      style: TextStyle(color: Colors.white),
                      autofocus: false,
                      decoration: InputDecoration(
                        labelText: "用户名",
                        hintText: "请输入用户名",
                        labelStyle: TextStyle(color: Colors.white),
                        hintStyle: TextStyle(
                            color: Color.fromRGBO(255, 255, 255, 0.5)),
                        border: InputBorder.none,
                      ),
                      keyboardType: TextInputType.number,
                      onChanged: (value) => this.userName = value),
                ],
              ),
            ),
            Container(
              padding: EdgeInsets.symmetric(horizontal: 20),
              child: Column(
                children: [
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    title: Text("开启摄像头", style: TextStyle(color: Colors.white)),
                    trailing: Switch(
                      value: enabledCamera,
                      onChanged: (value) =>
                          this.setState(() => enabledCamera = value),
                    ),
                  ),
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    title: Text("开启麦克风", style: TextStyle(color: Colors.white)),
                    trailing: Switch(
                      value: enabledMicrophone,
                      onChanged: (value) =>
                          this.setState(() => enabledMicrophone = value),
                    ),
                  ),
                  ListTile(
                    contentPadding: EdgeInsets.all(0),
                    title: Text("音质选择", style: TextStyle(color: Colors.white)),
                    subtitle: Row(
                      children: [
                        {
                          "text": "语音",
                          "value": TRTCCloudDef.TRTC_AUDIO_QUALITY_SPEECH
                        },
                        {
                          "text": "标准",
                          "value": TRTCCloudDef.TRTC_AUDIO_QUALITY_DEFAULT
                        },
                        {
                          "text": "音乐",
                          "value": TRTCCloudDef.TRTC_AUDIO_QUALITY_MUSIC
                        },
                      ]
                          .map(
                            (e) => Expanded(
                              child: Row(
                                children: [
                                  Theme(
                                    data: ThemeData(
                                        unselectedWidgetColor:
                                            Color.fromRGBO(102, 102, 102, 1)),
                                    child: Radio(
                                      hoverColor: Colors.white,
                                      value: e["value"],
                                      groupValue: this.quality,
                                      onChanged: (value) => this
                                          .setState(() => this.quality = value),
                                    ),
                                  ),
                                  Text(e["text"],
                                      style: TextStyle(color: Colors.white)),
                                ],
                              ),
                            ),
                          )
                          .toList(),
                    ),
                  ),
                ],
              ),
            ),
            Padding(
              padding: const EdgeInsets.only(top: 30.0),
              child: Row(
                children: <Widget>[
                  Expanded(
                    child: RaisedButton(
                      padding: EdgeInsets.all(15.0),
                      child: Text("进入会议"),
                      color: Theme.of(context).primaryColor,
                      textColor: Colors.white,
                      onPressed: enterMeeting,
                    ),
                  ),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }
}
