import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';
import './setting.dart';

import 'package:trtc_cloud_plugin/TRTCCloudVideoView.dart';
import 'package:trtc_cloud_plugin/TRTCCloud.dart';
import 'package:trtc_cloud_plugin/TXBeautyManager.dart';
import 'package:trtc_cloud_plugin/TXDeviceManager.dart';
import 'package:trtc_cloud_plugin/TXAudioEffectManager.dart';
import 'package:trtc_cloud_plugin/TRTCCloudDef.dart';
import 'package:trtc_cloud_plugin/TRTCCloudListener.dart';
import 'package:trtc_cloud_plugin_example/page/trtcmeetingdemo/index.dart';
import 'package:trtc_cloud_plugin_example/models/meeting.dart';
import 'package:trtc_cloud_plugin_example/debug/GenerateTestUserSig.dart';
import 'package:provider/provider.dart';

/// 视频页面
class MeetingPage extends StatefulWidget {
  @override
  State<StatefulWidget> createState() => MeetingPageState();
}

class MeetingPageState extends State<MeetingPage> {
  final _scaffoldKey = GlobalKey<ScaffoldState>();
  static const int SDKAPP_ID = 1400188366;
  var meetModel;
  var userInfo;

  bool isOpenMic = true; //是否开启麦克风
  bool isOpenCamera = true; //是否开启摄像头
  bool isFrontCamera = true; //是否是前置摄像头
  bool isSpeak = true; //是否是扬声器

  bool isShowBeauty = true; //是否开启美颜设置
  String curBeauty = 'pitu'; //默认为P图
  double curBeautyValue = 6; // 美颜值默认为6

  /// 本地控制器
  TRTCCloudVideoViewController trtcLocalController;
  TRTCCloud trtcCloud;
  TXDeviceManager txDeviceManager;
  TXBeautyManager txBeautyManager;
  TXAudioEffectManager txAudioManager;

  List userList = [];
  List screenUserList = [];
  int meetId;
  String userName;
  int quality;
  @override
  initState() {
    super.initState();
    SchedulerBinding.instance.addPostFrameCallback((_) {
      Map arguments =
          ModalRoute.of(_scaffoldKey.currentContext).settings.arguments;
      meetId = arguments["meetId"];
      userName = arguments["userName"];
      isOpenCamera = arguments["enabledCamera"];
      isOpenMic = arguments["enabledMicrophone"];
      quality = arguments["quality"];
    });
    meetModel = context.read<MeetingModel>();
    userInfo = meetModel.getUserInfo();
    iniRoom();
  }

  iniRoom() async {
    var aa = await GenerateTestUserSig.genTestSig('122');
    print('==aa = ' + aa.toString());
    trtcCloud = await TRTCCloud.sharedInstance();
    txDeviceManager = trtcCloud.getDeviceManager();
    txBeautyManager = trtcCloud.getBeautyManager();
    txAudioManager = trtcCloud.getAudioEffectManager();
    enterRoom();
    trtcCloud.setLogLevel(4);
    trtcCloud.setConsoleEnabled(true);
    trtcCloud.setListener(onRtcListener);
    if (isOpenCamera) {
      userList.add(
          {'userId': userInfo['userId'], 'type': 'video', 'visible': true});
    } else {
      userList.add(
          {'userId': userInfo['userId'], 'type': 'video', 'visible': false});
    }
    if (isOpenMic) {
      await trtcCloud.startLocalAudio(quality);
    }
    screenUserList = getScreenList(userList);
    meetModel.setList(userList);
    this.setState(() {});
    txBeautyManager.setBeautyStyle(TRTCCloudDef.TRTC_BEAUTY_STYLE_PITU);
  }

  // 进入房间(加入或创建)
  enterRoom() async {
    TRTCParams trtcParam = new TRTCParams();
    trtcParam.sdkAppId = GenerateTestUserSig.sdkAppId;
    trtcParam.userId = userInfo['userId'];
    trtcParam.userSig = userInfo['userSig'];
    // trtcParam.userSig = await GenerateTestUserSig.genTestSig('122');
    trtcParam.role = TRTCCloudDef.TRTC_APP_SCENE_LIVE;
    trtcParam.roomId = meetId;
    trtcCloud.enterRoom(trtcParam, TRTCCloudDef.TRTC_APP_SCENE_VIDEOCALL);
  }

  // 销毁房间的一些信息
  destoryRoom() {
    trtcCloud.removeListener(onRtcListener);
    trtcCloud.exitRoom();
    TRTCCloud.destroySharedInstance();
  }

  @override
  dispose() {
    destoryRoom();
    super.dispose();
  }

  // 设置混流
  setMixConfig() {
    trtcCloud.setMixTranscodingConfig(TRTCTranscodingConfig(
      appId: 1252463788,
      bizId: 3891,
      videoWidth: 360,
      mode: TRTCCloudDef.TRTC_VIDEO_RESOLUTION_MODE_PORTRAIT,
      videoHeight: 640,
      videoFramerate: 15,
      videoGOP: 2,
      videoBitrate: 1000,
      audioBitrate: 64,
      audioSampleRate: 48000,
      audioChannels: 2,
      streamId: null,
      mixUsers: [
        TRTCMixUser(
            userId: '57890',
            roomId: '256',
            zOrder: 1,
            x: 0,
            y: 0,
            streamType: 0,
            width: 300,
            height: 400),
        TRTCMixUser(
            userId: '345',
            roomId: '256',
            zOrder: 3,
            x: 100,
            y: 100,
            streamType: 0,
            width: 160,
            height: 200)
      ],
    ));
  }

  /// RTC监听器
  onRtcListener(type, param) {
    if (type == ListenerTypeEnum.onError) {
      showErrordDialog(param['errMsg']);
    }
    if (type == ListenerTypeEnum.onEnterRoom) {
      print('==trtc local onEnterRoom ' + param.toString());

      // setMixConfig();
    }
    if (type == ListenerTypeEnum.onSetMixTranscodingConfig) {
      print('==trtc local onSetMixTranscodingConfig ' + param.toString());
    }

    // 用户上传视频监听
    if (type == ListenerTypeEnum.onRemoteUserEnterRoom) {
      print('==trtc onRemoteUserEnterRoom ' + param.toString());
      userList.add({'userId': param, 'type': 'video', 'visible': false});
      screenUserList = getScreenList(userList);
      this.setState(() {});
      meetModel.setList(userList);
    }
    if (type == ListenerTypeEnum.onRemoteUserLeaveRoom) {
      print('==trtc onRemoteUserLeaveRoom ' + param.toString());
      String userId = param['userId'];
      for (var i = 0; i < userList.length; i++) {
        if (userList[i]['userId'] == userId) {
          trtcCloud.stopRemoteView(
              userList[i]['userId'], TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SMALL);
          userList.removeAt(i);
        }
      }
      screenUserList = getScreenList(userList);
      this.setState(() {});
      meetModel.setList(userList);
    }

    if (type == ListenerTypeEnum.onUserVideoAvailable) {
      print('==trtc onUserVideoAvailable ' + param.toString());
      String userId = param['userId'];
      // 根据状态对视频进行开启和关闭
      if (param['available']) {
        for (var i = 0; i < userList.length; i++) {
          if (userList[i]['userId'] == userId &&
              userList[i]['type'] == 'video') {
            userList[i]['visible'] = true;
          }
        }
      } else {
        for (var i = 0; i < userList.length; i++) {
          if (userList[i]['userId'] == userId &&
              userList[i]['type'] == 'video') {
            trtcCloud.stopRemoteView(
                userId, TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SMALL);
            userList[i]['visible'] = false;
          }
        }
      }
      screenUserList = getScreenList(userList);
      this.setState(() {});
      meetModel.setList(userList);
    }

    //辅流监听
    if (type == ListenerTypeEnum.onUserSubStreamAvailable) {
      print('==trtc userList subStream ' + param.toString());
      String userId = param["userId"];
      //视频可用
      if (param["available"]) {
        userList.add({'userId': userId, 'type': 'subStream', 'visible': true});
      } else {
        for (var i = 0; i < userList.length; i++) {
          if (userList[i]['userId'] == userId &&
              userList[i]['type'] == 'subStream') {
            trtcCloud.stopRemoteView(
                userId, TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB);
            userList.removeAt(i);
          }
        }
      }
      screenUserList = getScreenList(userList);
      this.setState(() {});
      meetModel.setList(userList);
    }
  }

  // 每4个一屏，得到一个二维数组
  getScreenList(list) {
    int len = 4; //4个一屏
    List<List> result = List();
    int index = 1;
    while (true) {
      if (index * len < list.length) {
        List temp = list.skip((index - 1) * len).take(len).toList();
        result.add(temp);
        index++;
        continue;
      }
      List temp = list.skip((index - 1) * len).toList();
      result.add(temp);
      break;
    }
    return result;
  }

  dealUserCreated(item, controller) {
    print('==trtc dealUser item ' + item.toString());
    if (item['userId'] == userInfo['userId']) {
      //本人
      trtcLocalController = controller;
      trtcLocalController.startLocalPreview(true);
    } else {
      //远端用户
      if (item['type'] == 'video') {
        trtcCloud.stopRemoteView(
            item['userId'], TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SMALL);
        controller.startRemoteView(
            item['userId'], TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SMALL);
      } else {
        controller.startRemoteView(
            item['userId'], TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_SUB);
      }
    }
  }

  /// 获得视图宽高
  Size getViewSize(int listLength, int index, int total) {
    Size screenSize = MediaQuery.of(context).size;

    if (listLength < 5) {
      // 只有一个显示全屏
      if (total == 1) {
        return screenSize;
      }
      // 两个显示半屏
      if (total == 2) {
        return Size(screenSize.width, screenSize.height / 2);
      }
    }
    return Size(screenSize.width / 2, screenSize.height / 2);
  }

  /// 用户列表点击事件
  getRoomList() {
    showModalBottomSheet(
        builder: (BuildContext context) {
          return ListView(
            children: ListTile.divideTiles(
              context: context,
              tiles: userList
                  .map(
                    (item) => ListTile(
                      title: Text(item['userId']),
                    ),
                  )
                  .toList(),
            ).toList(),
          );
        },
        context: context);
  }

  // sdk出错信查看
  Future<bool> showErrordDialog(errorMsg) {
    return showDialog<bool>(
      context: context,
      barrierDismissible: false,
      builder: (context) {
        return AlertDialog(
          title: Text("提示"),
          content: Text(errorMsg),
          actions: <Widget>[
            FlatButton(
              child: Text("确定"),
              onPressed: () {
                //关闭对话框并返回true
                Navigator.push(
                  context,
                  MaterialPageRoute(
                    builder: (context) => IndexPage(),
                  ),
                );
              },
            ),
          ],
        );
      },
    );
  }

  // 弹出对话框
  Future<bool> showExitMeetingConfirmDialog() {
    return showDialog<bool>(
      context: context,
      builder: (context) {
        return AlertDialog(
          title: Text("提示"),
          content: Text("确定退出会议?"),
          actions: <Widget>[
            FlatButton(
              child: Text("取消"),
              onPressed: () => Navigator.of(context).pop(), // 关闭对话框
            ),
            FlatButton(
              child: Text("确定"),
              onPressed: () {
                //关闭对话框并返回true
                Navigator.of(context).pop(true);
              },
            ),
          ],
        );
      },
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      key: _scaffoldKey,
      body: WillPopScope(
        onWillPop: () async {
          trtcCloud.exitRoom();
          return true;
        },
        child: Stack(
          children: <Widget>[
            ListView(
              // This next line does the trick.
              scrollDirection: Axis.horizontal,
              children: screenUserList
                  .map<Widget>(
                    (item) => Container(
                      width: MediaQuery.of(context).size.width,
                      height: MediaQuery.of(context).size.height,
                      color: Color.fromRGBO(19, 41, 75, 1),
                      child: Wrap(
                        children: List.generate(
                          item.length,
                          (index) => LayoutBuilder(
                            key: ValueKey(
                                item[index]['userId'] + item[index]['type']),
                            builder: (BuildContext context,
                                BoxConstraints constraints) {
                              Size size = this.getViewSize(
                                  userList.length, index, item.length);
                              ValueKey valueKey = ValueKey(
                                  item[index]['userId'] + item[index]['type']);

                              return Container(
                                key: valueKey,
                                height: size.height,
                                width: size.width,
                                child: Stack(
                                  key: valueKey,
                                  children: <Widget>[
                                    item[index]['visible']
                                        ? TRTCCloudVideoView(
                                            key: valueKey,
                                            onViewCreated: (controller) {
                                              dealUserCreated(
                                                  item[index], controller);
                                            },
                                          )
                                        : Container(
                                            alignment: Alignment.center,
                                            child: ClipOval(
                                              child: Image.network(
                                                  'https://imgcache.qq.com/qcloud/public/static//avatar3_100.20191230.png',
                                                  scale: 3.5),
                                            ),
                                          ),
                                    Positioned(
                                      // red box
                                      child: new Container(
                                          child: Row(children: <Widget>[
                                        Text(
                                          item[index]['userId'] ==
                                                  userInfo['userId']
                                              ? userName
                                              : item[index]['userId'],
                                          style: TextStyle(color: Colors.white),
                                        ),
                                        Container(
                                          margin: EdgeInsets.only(left: 10),
                                          child: Icon(
                                            Icons.signal_cellular_alt,
                                            color: Colors.white,
                                            size: 20,
                                          ),
                                        ),
                                      ])),
                                      left: 24.0,
                                      bottom: 55.0,
                                    ),
                                  ],
                                ),
                              );
                            },
                          ),
                        ),
                      ),
                    ),
                  )
                  .toList(),
            ),
            new Align(
                child: new Container(
                  margin:
                      EdgeInsets.only(top: MediaQuery.of(context).padding.top),
                  // grey box
                  child: new Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: <Widget>[
                      IconButton(
                          icon: Icon(
                            isSpeak ? Icons.volume_up : Icons.hearing,
                            color: Colors.white,
                            size: 36.0,
                          ),
                          onPressed: () async {
                            if (isSpeak) {
                              trtcCloud.setAudioRoute(1);

                              // int number =
                              //     await txDeviceManager.getCameraZoomMaxRatio();
                              // print('==trtc number=' + number.toString());

                              // trtcCloud.setRemoteRenderParams(
                              //     '345',
                              //     TRTCCloudDef.TRTC_VIDEO_STREAM_TYPE_BIG,
                              //     TRTCRenderParams(
                              //         rotation:
                              //             TRTCCloudDef.TRTC_VIDEO_ROTATION_90,
                              //         fillMode: TRTCCloudDef
                              //             .TRTC_VIDEO_RENDER_MODE_FIT,
                              //         mirrorType: TRTCCloudDef
                              //             .TRTC_VIDEO_MIRROR_TYPE_ENABLE));
                            } else {
                              trtcCloud.setAudioRoute(0);
                            }
                            setState(() {
                              isSpeak = !isSpeak;
                            });
                          }),
                      IconButton(
                          icon: Icon(
                            Icons.camera_alt,
                            color: Colors.white,
                            size: 36.0,
                          ),
                          onPressed: () async {
                            if (isFrontCamera) {
                              trtcCloud.stopLocalPreview();
                              trtcLocalController.startLocalPreview(false);
                            } else {
                              trtcCloud.stopLocalPreview();
                              trtcLocalController.startLocalPreview(true);
                            }
                            setState(() {
                              isFrontCamera = !isFrontCamera;
                            });
                          }),
                      Text(meetId.toString(),
                          style: TextStyle(fontSize: 20, color: Colors.white)),
                      FlatButton(
                        color: Colors.red,
                        textColor: Colors.white,
                        onPressed: () async {
                          //弹出对话框并等待其关闭
                          bool delete = await showExitMeetingConfirmDialog();
                          if (delete != null) {
                            trtcCloud.exitRoom();
                            Navigator.push(
                              context,
                              MaterialPageRoute(
                                builder: (context) => IndexPage(),
                              ),
                            );
                          }
                        },
                        child: Text(
                          "退出会议",
                          style: TextStyle(fontSize: 16.0),
                        ),
                      )
                    ],
                  ),
                  height: 50.0,
                  color: Color.fromRGBO(200, 200, 200, 0.4),
                ),
                alignment: Alignment.topCenter),
            Positioned(
              bottom: 50,
              child: Offstage(
                offstage: isShowBeauty,
                child: Container(
                  padding: EdgeInsets.all(10),
                  color: Color.fromRGBO(0, 0, 0, 0.8),
                  height: 100,
                  width: MediaQuery.of(context).size.width,
                  child: Column(
                    children: [
                      Row(children: [
                        Expanded(
                          flex: 2,
                          child: Slider(
                            value: curBeautyValue,
                            min: 0,
                            max: 9,
                            divisions: 9,
                            onChanged: (double value) {
                              if (curBeauty == 'smooth' ||
                                  curBeauty == 'nature' ||
                                  curBeauty == 'pitu') {
                                txBeautyManager.setBeautyLevel(value.round());
                              } else if (curBeauty == 'whitening') {
                                txBeautyManager
                                    .setWhitenessLevel(value.round());
                              } else if (curBeauty == 'ruddy') {
                                txBeautyManager.setRuddyLevel(value.round());
                              }
                              this.setState(() {
                                curBeautyValue = value;
                              });
                            },
                          ),
                        ),
                        Text(curBeautyValue.round().toString(),
                            textAlign: TextAlign.center,
                            style: TextStyle(color: Colors.white)),
                      ]),
                      Padding(
                        padding: EdgeInsets.only(top: 10),
                        child: Row(
                          children: [
                            GestureDetector(
                              child: Container(
                                alignment: Alignment.centerLeft,
                                width: 80.0,
                                child: Text(
                                  '美颜(光滑)',
                                  style: TextStyle(
                                      color: curBeauty == 'smooth'
                                          ? Color.fromRGBO(64, 158, 255, 1)
                                          : Colors.white),
                                ),
                              ),
                              onTap: () => this.setState(() {
                                txBeautyManager.setBeautyStyle(
                                    TRTCCloudDef.TRTC_BEAUTY_STYLE_SMOOTH);
                                curBeauty = 'smooth';
                                curBeautyValue = 6;
                              }),
                            ),
                            GestureDetector(
                              child: Container(
                                alignment: Alignment.centerLeft,
                                width: 80.0,
                                child: Text(
                                  '美颜(自然)',
                                  style: TextStyle(
                                      color: curBeauty == 'nature'
                                          ? Color.fromRGBO(64, 158, 255, 1)
                                          : Colors.white),
                                ),
                              ),
                              onTap: () => this.setState(() {
                                txBeautyManager.setBeautyStyle(
                                    TRTCCloudDef.TRTC_BEAUTY_STYLE_NATURE);
                                curBeauty = 'nature';
                                curBeautyValue = 6;
                              }),
                            ),
                            GestureDetector(
                              child: Container(
                                alignment: Alignment.centerLeft,
                                width: 80.0,
                                child: Text(
                                  '美颜(P图)',
                                  style: TextStyle(
                                      color: curBeauty == 'pitu'
                                          ? Color.fromRGBO(64, 158, 255, 1)
                                          : Colors.white),
                                ),
                              ),
                              onTap: () => this.setState(() {
                                txBeautyManager.setBeautyStyle(
                                    TRTCCloudDef.TRTC_BEAUTY_STYLE_PITU);
                                curBeauty = 'pitu';
                                curBeautyValue = 6;
                              }),
                            ),
                            GestureDetector(
                              child: Container(
                                alignment: Alignment.centerLeft,
                                width: 50.0,
                                child: Text(
                                  '美白',
                                  style: TextStyle(
                                      color: curBeauty == 'whitening'
                                          ? Color.fromRGBO(64, 158, 255, 1)
                                          : Colors.white),
                                ),
                              ),
                              onTap: () => this.setState(() {
                                txBeautyManager.setWhitenessLevel(0);
                                curBeauty = 'whitening';
                                curBeautyValue = 0;
                              }),
                            ),
                            GestureDetector(
                              child: Container(
                                alignment: Alignment.centerLeft,
                                width: 50.0,
                                child: Text(
                                  '红润',
                                  style: TextStyle(
                                      color: curBeauty == 'ruddy'
                                          ? Color.fromRGBO(64, 158, 255, 1)
                                          : Colors.white),
                                ),
                              ),
                              onTap: () => this.setState(() {
                                txBeautyManager.setRuddyLevel(0);
                                curBeauty = 'ruddy';
                                curBeautyValue = 0;
                              }),
                            ),
                          ],
                        ),
                      ),
                    ],
                  ),
                ),
              ),
            ),
            new Align(
                child: new Container(
                  // grey box
                  child: new Row(
                    mainAxisAlignment: MainAxisAlignment.spaceAround,
                    children: <Widget>[
                      IconButton(
                          icon: Icon(
                            isOpenMic ? Icons.mic : Icons.mic_off,
                            color: Colors.white,
                            size: 36.0,
                          ),
                          onPressed: () {
                            if (isOpenMic) {
                              trtcCloud.stopLocalAudio();
                            } else {
                              trtcCloud.startLocalAudio(quality);
                            }
                            setState(() {
                              isOpenMic = !isOpenMic;
                            });
                          }),
                      IconButton(
                          icon: Icon(
                            isOpenCamera ? Icons.videocam : Icons.videocam_off,
                            color: Colors.white,
                            size: 36.0,
                          ),
                          onPressed: () {
                            if (isOpenCamera) {
                              userList[0]['visible'] = false;
                              trtcCloud.stopLocalPreview();
                            } else {
                              userList[0]['visible'] = true;
                            }
                            setState(() {
                              isOpenCamera = !isOpenCamera;
                            });
                          }),
                      IconButton(
                          icon: Icon(
                            Icons.face,
                            color: Colors.white,
                            size: 36.0,
                          ),
                          onPressed: () {
                            this.setState(() {
                              if (isShowBeauty) {
                                isShowBeauty = false;
                              } else {
                                isShowBeauty = true;
                              }
                            });
                          }),
                      IconButton(
                          icon: Icon(
                            Icons.people,
                            color: Colors.white,
                            size: 36.0,
                          ),
                          onPressed: () {
                            // getRoomList();
                            Navigator.pushNamed(context, '/memberList');
                          }),
                      SettingPage()
                    ],
                  ),
                  height: 50.0,
                  color: Color.fromRGBO(200, 200, 200, 0.4),
                ),
                alignment: Alignment.bottomCenter),
          ],
        ),
      ),
    );
  }
}
