import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class TXAudioEffectManager {
  static MethodChannel _channel;
  TXAudioEffectManager(channel) {
    _channel = channel;
  }

  /// 开启耳返
  ///
  /// 开启后会在耳机里听到自己的声音。
  Future<void> enableVoiceEarMonitor(bool enable) async {
    return await _channel
        .invokeMethod('enableVoiceEarMonitor', {"enable": enable});
  }

  /// 设置耳返音量。
  ///
  /// volume	音量大小，取值0 - 100，默认值为100
  Future<void> setVoiceEarMonitorVolume(int volume) async {
    return await _channel
        .invokeMethod('setVoiceEarMonitorVolume', {"volume": volume});
  }

  /// 设置人声的混响效果（KTV、小房间、大会堂、低沉、洪亮...）
  Future<void> setVoiceReverbType(int type) async {
    return await _channel.invokeMethod('setVoiceReverbType', {"type": type});
  }

  /// 设置人声的变声特效（萝莉、大叔、重金属、外国人...）
  Future<void> setVoiceChangerType(int type) async {
    return await _channel.invokeMethod('setVoiceChangerType', {"type": type});
  }

  /// 设置麦克风采集人声的音量
  ///
  /// volume	音量大小，1为正常音量，范围是：[0 ~ 100] 之间的浮点数
  Future<void> setVoiceCaptureVolume(int volume) async {
    return await _channel
        .invokeMethod('setVoiceCaptureVolume', {"volume": volume});
  }

  /// 设置背景音乐的播放进度
  Future<void> setMusicObserver(int id) async {
    return await _channel.invokeMethod('setMusicObserver', {"id": id});
  }

  /// 开始播放背景音乐
  ///
  /// 每个音乐都需要您指定具体的 ID，您可以通过该 ID 对音乐的开始、停止、音量等进行设置。
  Future<bool> startPlayMusic(
      {@required int id,
      @required String path,
      int loopCount,
      bool publish,
      bool isShortFile,
      int startTimeMS,
      int endTimeMS}) async {
    return await _channel.invokeMethod('startPlayMusic', {
      "id": id,
      "path": path,
      "loopCount": loopCount == null ? 0 : loopCount,
      "publish": publish == null ? false : publish,
      "isShortFile": isShortFile == null ? false : isShortFile,
      "startTimeMS": startTimeMS == null ? '0' : startTimeMS.toString(),
      "endTimeMS": endTimeMS == null ? '-1' : endTimeMS.toString()
    });
  }

  /// 停止播放背景音乐
  Future<void> stopPlayMusic(int id) async {
    return await _channel.invokeMethod('stopPlayMusic', {"id": id});
  }

  /// 暂停播放背景音乐
  Future<void> pausePlayMusic(int id) async {
    return await _channel.invokeMethod('pausePlayMusic', {"id": id});
  }

  /// 恢复播放背景音乐
  Future<void> resumePlayMusic(int id) async {
    return await _channel.invokeMethod('resumePlayMusic', {"id": id});
  }

  /// 设置背景音乐的远端音量大小，即主播可以通过此接口设置远端观众能听到的背景音乐的音量大小。
  Future<void> setMusicPublishVolume(int id, int volume) async {
    return await _channel
        .invokeMethod('setMusicPublishVolume', {"id": id, "volume": volume});
  }

  /// 设置背景音乐的本地音量大小，即主播可以通过此接口设置主播自己本地的背景音乐的音量大小。
  Future<void> setMusicPlayoutVolume(int id, int volume) async {
    return await _channel
        .invokeMethod('setMusicPlayoutVolume', {"id": id, "volume": volume});
  }

  /// 设置全局背景音乐的本地和远端音量的大小
  Future<void> setAllMusicVolume(int volume) async {
    return await _channel.invokeMethod('setAllMusicVolume', {"volume": volume});
  }

  /// 调整背景音乐的音调高低
  Future<void> setMusicPitch(int volume, double pitch) async {
    return await _channel.invokeMethod(
        'setMusicPitch', {"volume": volume, "pitch": pitch.toString()});
  }

  /// 调整背景音乐的变速效果
  Future<void> setMusicSpeedRate(int id, double speedRate) async {
    return await _channel.invokeMethod(
        'setMusicSpeedRate', {"id": id, "speedRate": speedRate.toString()});
  }

  /// 获取背景音乐当前的播放进度（单位：毫秒）
  Future<int> getMusicCurrentPosInMS(int id) async {
    return await _channel.invokeMethod('getMusicCurrentPosInMS', {"id": id});
  }

  /// 设置背景音乐的播放进度（单位：毫秒）
  Future<void> seekMusicToPosInMS(int id, int pts) async {
    return await _channel
        .invokeMethod('seekMusicToPosInMS', {"id": id, "pts": pts});
  }

  /// 获取景音乐文件的总时长（单位：毫秒）
  Future<int> getMusicDurationInMS(String path) async {
    return await _channel.invokeMethod('getMusicDurationInMS', {"path": path});
  }
}
