import 'dart:async';
import 'package:flutter/services.dart';
import 'util/CommonUtil.dart';

class TXBeautyManager {
  static MethodChannel _channel;
  TXBeautyManager(channel) {
    _channel = channel;
  }

  /// 设置美颜类型
  ///
  /// beautyStyle	美颜风格.三种美颜风格：0 ：光滑 1：自然 2：朦胧
  Future<void> setBeautyStyle(int beautyStyle) async {
    return _channel
        .invokeMethod('setBeautyStyle', {"beautyStyle": beautyStyle});
  }

  /// 设置指定素材滤镜特效
  ///
  /// image	指定素材，即颜色查找表图片。必须使用 png 格式
  Future<void> setFilter(String assetUrl //assets 中的资源地址
      ) async {
    String imageUrl = await CommonUtil.copyAssetToLocal(assetUrl);

    return _channel.invokeMethod('setFilter', {"imageUrl": imageUrl});
  }

  /// 设置滤镜浓度
  ///
  /// 在美女秀场等应用场景里，滤镜浓度的要求会比较高，以便更加突显主播的差异。 我们默认的滤镜浓度是0.5，如果您觉得滤镜效果不明显，可以使用下面的接口进行调节。
  Future<void> setFilterStrength(double strength) async {
    return await _channel
        .invokeMethod('setFilterStrength', {"strength": strength});
  }

  /// 设置美颜级别
  ///
  /// beautyLevel	美颜级别，取值范围0 - 9； 0表示关闭，1 - 9值越大，效果越明显。
  Future<void> setBeautyLevel(int beautyLevel) async {
    return await _channel
        .invokeMethod('setBeautyLevel', {"beautyLevel": beautyLevel});
  }

  /// 设置美白级别
  ///
  /// whitenessLevel	美白级别，取值范围0 - 9； 0表示关闭，1 - 9值越大，效果越明显。
  Future<void> setWhitenessLevel(int whitenessLevel) async {
    return await _channel
        .invokeMethod('setWhitenessLevel', {"whitenessLevel": whitenessLevel});
  }

  /// 开启清晰度增强
  ///
  /// enable	true：开启清晰度增强；false：关闭清晰度增强。默认值：true
  Future<void> enableSharpnessEnhancement(bool enable) async {
    return await _channel
        .invokeMethod('enableSharpnessEnhancement', {"enable": enable});
  }

  /// 设置红润级别
  ///
  /// ruddyLevel	红润级别，取值范围0 - 9； 0表示关闭，1 - 9值越大，效果越明显。
  Future<void> setRuddyLevel(int ruddyLevel) async {
    return await _channel
        .invokeMethod('setRuddyLevel', {"ruddyLevel": ruddyLevel});
  }
}
