import 'dart:async';
import 'package:flutter/services.dart';

class TXDeviceManager {
  static MethodChannel _channel;
  TXDeviceManager(channel) {
    _channel = channel;
  }

  // 摄像头相关接口函数

  /// 是否使用前置摄像头
  Future<bool> isFrontCamera() {
    return _channel.invokeMethod('isFrontCamera');
  }

  /// 切换摄像头。
  Future<void> switchCamera() {
    return _channel.invokeMethod('switchCamera');
  }

  /// 查询当前摄像头是否支持缩放。
  Future<bool> isCameraZoomSupported() {
    return _channel.invokeMethod('isCameraZoomSupported');
  }

  /// 获取摄像头的缩放因子
  Future<int> getCameraZoomMaxRatio() {
    return _channel.invokeMethod('getCameraZoomMaxRatio');
  }

  /// 设置摄像头缩放因子（焦距）。
  Future<void> setCameraZoomRatio(int value // 取值范围为1 - 5，数值越大，焦距越远。
      ) {
    return _channel.invokeMethod('setCameraZoomRatio', {
      "value": value,
    });
  }

  /// 查询是否支持开关闪光灯（手电筒模式）。
  Future<bool> isCameraTorchSupported() {
    return _channel.invokeMethod('isCameraTorchSupported');
  }

  /// 开关闪光灯。
  Future<void> enableCameraTorch(bool enable // true：开启；false：关闭，默认值：false。
      ) {
    return _channel.invokeMethod('enableCameraTorch', {
      "enable": enable,
    });
  }

  /// 查询是否支持设置焦点。
  Future<bool> isCameraFocusPositionInPreviewSupported() {
    return _channel.invokeMethod('isCameraFocusPositionInPreviewSupported');
  }

  /// 设置摄像头焦点。
  Future<void> setCameraFocusPosition(int x, int y) {
    return _channel.invokeMethod('setCameraFocusPosition', {
      "x": x,
      "y": y,
    });
  }

  /// 查询是否支持自动识别人脸位置。
  Future<bool> isCameraAutoFocusFaceModeSupported() {
    return _channel.invokeMethod('isCameraAutoFocusFaceModeSupported');
  }
}
